@@ -18,10 +18,12 @@ use Inline C => <<'END_OF_C_CODE';
  *
  */
 
+void __swap(AV* av, int i, int j);
 
-#define UPDATE(av, i, n) (sv_setiv(*av_fetch(av, i, 0), n))
+#define UPDATE(av, i, n)   (sv_setiv(*av_fetch(av, i, 0), n))
+#define GET(av, i)         (SvIVX(*av_fetch(av, i, 0)))
 
-int __next_combination(AV* indices, AV* data, AV* out)
+int __next_combination(AV* indices, int max_n)
 {
     int i, j;
     IV  n;
@@ -29,7 +31,7 @@ int __next_combination(AV* indices, AV* data, AV* out)
     SV* index;
 
     len_indices = av_len(indices);
-    offset = av_len(data) - len_indices;
+    offset = max_n - len_indices;
     for (i = len_indices; i >= 0; --i) {
         index = *av_fetch(indices, i, 0);
         n = SvIVX(index);
@@ -37,7 +39,6 @@ int __next_combination(AV* indices, AV* data, AV* out)
              sv_setiv(index, ++n);
              for (j = i+1; j <= len_indices; ++j)
                   UPDATE(indices, j, ++n);
-             __slice_and_copy(i, indices, data, out);
              return i;
         }
     }
@@ -45,22 +46,19 @@ int __next_combination(AV* indices, AV* data, AV* out)
     return -1;
 }
 
-
-int __next_combination_with_repetition(AV* indices, AV* data, AV* out)
+int __next_combination_with_repetition(AV* indices, int max_n)
 {
     int i, j;
     IV  n;
-    I32 len_indices, max_n;
+    I32 len_indices;
 
     len_indices = av_len(indices);
-    max_n = av_len(data);
     for (i = len_indices; i >= 0; --i) {
-        n = SvIVX(*av_fetch(indices, i, 0));
+        n = GET(indices, i);
         if (n < max_n) {
              ++n;
              for (j = i; j <= len_indices; ++j)
                   UPDATE(indices, j, n);
-             __slice_and_copy(i, indices, data, out);
              return i;
         }
     }
@@ -69,15 +67,14 @@ int __next_combination_with_repetition(AV* indices, AV* data, AV* out)
 }
 
 
-int __next_variation(HV* used, AV* indices, AV* data, AV* out)
+int __next_variation(HV* used, AV* indices, int max_n)
 {
     int i, j;
-    I32 len_indices, max_n;
+    I32 len_indices;
     SV* index;
     IV  n;
     HE* he;
 
-    max_n = av_len(data);
     len_indices = av_len(indices);
     for (i = len_indices; i >= 0; --i) {
         index = *av_fetch(indices, i, 0);
@@ -95,13 +92,12 @@ int __next_variation(HV* used, AV* indices, AV* data, AV* out)
                             ++n;
                             he = hv_fetch_ent(used, newSViv(n), 0, 0);
                             if (he == NULL) {
-	                             UPDATE(indices, j, n);
+                                 UPDATE(indices, j, n);
                                  hv_store_ent(used, newSViv(n), newSViv(j), 0);
                                  break;
                             }
                        }
                   }
-                  __slice_and_copy(i, indices, data, out);
                   return i;
              }
         }
@@ -111,19 +107,17 @@ int __next_variation(HV* used, AV* indices, AV* data, AV* out)
 }
 
 
-int __next_variation_with_repetition(AV* indices, AV* data, AV* out)
+int __next_variation_with_repetition(AV* indices, int max_n)
 {
     int i;
-    I32 len_indices, max_n;
+    I32 len_indices;
     SV* index;
 
-    max_n = av_len(data);
     len_indices = av_len(indices);
     for (i = len_indices; i >= 0; --i) {
         index = *av_fetch(indices, i, 0);
         if (SvIVX(index) < max_n) {
             sv_inc(index);
-             __slice_and_copy(i, indices, data, out);
             return i;
         }
         sv_setiv(index, 0);
@@ -133,53 +127,39 @@ int __next_variation_with_repetition(AV* indices, AV* data, AV* out)
 }
 
 /* Adapted from http://www.scielo.br/scielo.php?pid=S0104-65002001000200009&script=sci_arttext&tlng=en */
-int __next_permutation(AV* indices, AV* data, AV* out)
+int __next_permutation(AV* indices, int max_n)
 {
-	int i, j, k;
-	I32 max_n = av_len(data);
-	
-	i = max_n;
-	while (i > 0 && SvIVX(*av_fetch(indices, i-1, 0)) > SvIVX(*av_fetch(indices, i, 0)))
-		--i;
-	
-	if (i == 0)
-		return -1;
+    int i, j, k;
+    
+    i = max_n;
+    while (i > 0 && GET(indices, i-1) > GET(indices, i))
+        --i;
+    
+    if (i == 0)
+        return -1;
 
-    j = i + 1;	
-	while (j <= max_n && SvIVX(*av_fetch(indices, i-1, 0)) < SvIVX(*av_fetch(indices, j, 0)))
-		++j;
-	
-	__swap(indices, i-1, j-1);
-	
-	k = (max_n-i)/2;
-	for (j = 0; j <= k; j++)
-		__swap(indices, i+j, max_n-j);
-	
-	__slice_and_copy(i-1, indices, data, out);
-	
-	return 1;
+    j = i + 1;  
+    while (j <= max_n && GET(indices, i-1) < GET(indices, j))
+        ++j;
+    
+    __swap(indices, i-1, j-1);
+    
+    k = (max_n-i)/2;
+    for (j = 0; j <= k; j++)
+        __swap(indices, i+j, max_n-j);      
+    
+    return 1;
 }
 
 void __swap(AV* av, int i, int j)
 {
-	SV* svi = *av_fetch(av, i, 0);
-	SV* svj = *av_fetch(av, j, 0);
-	IV  tmp = SvIVX(svi);
-	sv_setiv(svi, SvIVX(svj));
-	sv_setiv(svj, tmp);
-}
-
-void __slice_and_copy(int from, AV* indices, AV* data, AV* out)
-{
-     int i;
-     I32 len_indices;
-     IV  n;
-
-     len_indices = av_len(indices);
-     for (i = from; i <= len_indices; ++i) {
-          n = SvIVX(*av_fetch(indices, i, 0));
-          av_store(out, i, newSVsv(*av_fetch(data, n, 0)));
-     }
+	IV tmp;
+	
+	if (i != j) {		
+        tmp = SvIVX(*av_fetch(av, i, 0));
+        UPDATE(av, i, SvIVX(*av_fetch(av, j, 0)));
+        UPDATE(av, j, tmp);
+    }
 }
 END_OF_C_CODE
 
